use anyhow::{bail, Result};
use std::{fs::File, io::Read, path::Path};

use super::syntax;

#[derive(Debug)]
pub enum ContentType {
	System(syntax::system::toml::SystemRoot),
	Ship(syntax::ship::toml::ShipRoot),
}

// TODO: check content without loading game
impl ContentType {
	pub fn try_parse(file_string: &str) -> Result<Option<Self>> {
		// TODO: More advanced parsing, read the whole top comment
		let first = match file_string.split_once("\n") {
			None => bail!("This file is empty."),
			Some((first, _)) => first,
		};
		let type_spec = first[1..].trim(); // Remove hash

		let type_spec = if type_spec.starts_with("content type: ") {
			type_spec[14..].to_owned()
		} else {
			bail!("No content type specified")
		};

		return Ok(match &type_spec[..] {
			"system" => Some(Self::System(toml::from_str(&file_string)?)),
			"ship" => Some(Self::Ship(toml::from_str(&file_string)?)),
			_ => bail!("Invalid content type `{}`", type_spec),
		});
	}

	pub fn from_path(path: &Path) -> Result<Option<Self>> {
		let mut file_string = String::new();
		let _ = File::open(path)?.read_to_string(&mut file_string);
		let file_string = file_string.trim();
		return Self::try_parse(&file_string);
	}
}
